#ifndef DSOLVER_H
#define DSOLVER_H

#include "vii.h"
#include "DFluxes.h"

namespace vii
{

/**
 * @brief Pseudo-time integration.
 * @author Paul Dechamps
 */
class VII_API Solver
{

private:
    double CFL0;               ///< Initial CFL number.
    unsigned int maxIt;        ///< Maximum number of iterations.
    double tol;                ///< Convergence tolerance.
    unsigned int itFrozenJac0; ///< Number of iterations between which the Jacobian is frozen.
    bool initSln;              ///< Flag to initialize the solution at the point.
    double const gamma = 1.4;  ///< Air heat capacity ratio.
    double Minf;               ///< Freestream Mach number.

    Fluxes *sysMatrix;

public:
    Solver(double _CFL0, double _Minf, double Re, unsigned int _maxIt = 100, double _tol = 1e-8, unsigned int _itFrozenJac = 5);
    ~Solver();
    void initialCondition(size_t iPoint, BoundaryLayer *bl);
    int integration(size_t iPoint, BoundaryLayer *bl);

    // Getters.
    double getCFL0() const { return CFL0; };
    unsigned int getmaxIt() const { return maxIt; };
    unsigned int getitFrozenJac() const { return itFrozenJac0; };

    // Setters.
    void setCFL0(double _CFL0);
    void setmaxIt(unsigned int _maxIt) { maxIt = _maxIt; };
    void setitFrozenJac(unsigned int _itFrozenJac) { itFrozenJac0 = _itFrozenJac; };
    void setinitSln(bool _initSln) { initSln = _initSln; };

private:
    double setTimeStep(double CFL, double dx, double invVel) const;
    double computeSoundSpeed(double gradU2) const;
    void resetSolution(size_t iPoint, BoundaryLayer *bl, std::vector<double> Sln0, bool usePrevPoint = false);
};
} // namespace vii
#endif // DSOLVER_H
